; Core def : Declares the module parameters for the DLL.

EXPORTS
CreateOptionsBC1
CreateOptionsBC2
CreateOptionsBC3
CreateOptionsBC4
CreateOptionsBC5
CreateOptionsBC6
CreateOptionsBC7

DestroyOptionsBC1
DestroyOptionsBC2
DestroyOptionsBC3
DestroyOptionsBC4
DestroyOptionsBC5
DestroyOptionsBC6
DestroyOptionsBC7

SetDecodeChannelMapping

SetChannelWeightsBC1
SetChannelWeightsBC2
SetChannelWeightsBC3

SetQualityBC1
SetQualityBC2
SetQualityBC3
SetQualityBC4
SetQualityBC5
SetQualityBC6
SetQualityBC7

SetAlphaThresholdBC1

SetMaskBC6
SetMaskBC7

SetErrorThresholdBC7
SetAlphaOptionsBC7

CompressBlockBC1
CompressBlockBC2
CompressBlockBC3
CompressBlockBC4
CompressBlockBC5
CompressBlockBC6
CompressBlockBC7

DecompressBlockBC1
DecompressBlockBC2
DecompressBlockBC3
DecompressBlockBC4
DecompressBlockBC5
DecompressBlockBC6
DecompressBlockBC7
